@@ -10,9 +10,7 @@
 from __future__ import absolute_import
 
 from mercurial.i18n import _
-from mercurial.node import (
-    short,
-)
+from mercurial.node import short
 from mercurial import (
     cmdutil,
     error,
@@ -32,18 +30,30 @@ command = registrar.command(cmdtable)
 # extensions which SHIP WITH MERCURIAL. Non-mainline extensions should
 # be specifying the version(s) of Mercurial they are tested with, or
 # leave the attribute unspecified.
-testedwith = 'ships-with-hg-core'
+testedwith = b'ships-with-hg-core'
 
-@command('fetch',
-    [('r', 'rev', [],
-     _('a specific revision you would like to pull'), _('REV')),
-    ('', 'edit', None, _('invoke editor on commit messages')),
-    ('', 'force-editor', None, _('edit commit message (DEPRECATED)')),
-    ('', 'switch-parent', None, _('switch parents when merging')),
-    ] + cmdutil.commitopts + cmdutil.commitopts2 + cmdutil.remoteopts,
-    _('hg fetch [SOURCE]'),
-    helpcategory=command.CATEGORY_REMOTE_REPO_MANAGEMENT)
-def fetch(ui, repo, source='default', **opts):
+
+@command(
+    b'fetch',
+    [
+        (
+            b'r',
+            b'rev',
+            [],
+            _(b'a specific revision you would like to pull'),
+            _(b'REV'),
+        ),
+        (b'', b'edit', None, _(b'invoke editor on commit messages')),
+        (b'', b'force-editor', None, _(b'edit commit message (DEPRECATED)')),
+        (b'', b'switch-parent', None, _(b'switch parents when merging')),
+    ]
+    + cmdutil.commitopts
+    + cmdutil.commitopts2
+    + cmdutil.remoteopts,
+    _(b'hg fetch [SOURCE]'),
+    helpcategory=command.CATEGORY_REMOTE_REPO_MANAGEMENT,
+)
+def fetch(ui, repo, source=b'default', **opts):
     '''pull changes from a remote repository, merge new changes if needed.
 
     This finds all changes from the repository at the specified path
@@ -64,9 +74,9 @@ def fetch(ui, repo, source='default', **opts):
     '''
 
     opts = pycompat.byteskwargs(opts)
-    date = opts.get('date')
+    date = opts.get(b'date')
     if date:
-        opts['date'] = dateutil.parsedate(date)
+        opts[b'date'] = dateutil.parsedate(date)
 
     parent = repo.dirstate.p1()
     branch = repo.dirstate.branch()
@@ -75,8 +85,10 @@ def fetch(ui, repo, source='default', **opts):
     except error.RepoLookupError:
         branchnode = None
     if parent != branchnode:
-        raise error.Abort(_('working directory not at branch tip'),
-                         hint=_("use 'hg update' to check out branch tip"))
+        raise error.Abort(
+            _(b'working directory not at branch tip'),
+            hint=_(b"use 'hg update' to check out branch tip"),
+        )
 
     wlock = lock = None
     try:
@@ -88,19 +100,26 @@ def fetch(ui, repo, source='default', **opts):
         bheads = repo.branchheads(branch)
         bheads = [head for head in bheads if len(repo[head].children()) == 0]
         if len(bheads) > 1:
-            raise error.Abort(_('multiple heads in this branch '
-                               '(use "hg heads ." and "hg merge" to merge)'))
+            raise error.Abort(
+                _(
+                    b'multiple heads in this branch '
+                    b'(use "hg heads ." and "hg merge" to merge)'
+                )
+            )
 
         other = hg.peer(repo, opts, ui.expandpath(source))
-        ui.status(_('pulling from %s\n') %
-                  util.hidepassword(ui.expandpath(source)))
+        ui.status(
+            _(b'pulling from %s\n') % util.hidepassword(ui.expandpath(source))
+        )
         revs = None
-        if opts['rev']:
+        if opts[b'rev']:
             try:
-                revs = [other.lookup(rev) for rev in opts['rev']]
+                revs = [other.lookup(rev) for rev in opts[b'rev']]
             except error.CapabilityError:
-                err = _("other repository doesn't support revision lookup, "
-                        "so a rev cannot be specified.")
+                err = _(
+                    b"other repository doesn't support revision lookup, "
+                    b"so a rev cannot be specified."
+                )
                 raise error.Abort(err)
 
         # Are there any changes at all?
@@ -125,9 +144,13 @@ def fetch(ui, repo, source='default', **opts):
             hg.clean(repo, newparent)
         newheads = [n for n in newheads if n != newparent]
         if len(newheads) > 1:
-            ui.status(_('not merging with %d other new branch heads '
-                        '(use "hg heads ." and "hg merge" to merge them)\n') %
-                      (len(newheads) - 1))
+            ui.status(
+                _(
+                    b'not merging with %d other new branch heads '
+                    b'(use "hg heads ." and "hg merge" to merge them)\n'
+                )
+                % (len(newheads) - 1)
+            )
             return 1
 
         if not newheads:
@@ -139,29 +162,35 @@ def fetch(ui, repo, source='default', **opts):
             # By default, we consider the repository we're pulling
             # *from* as authoritative, so we merge our changes into
             # theirs.
-            if opts['switch_parent']:
+            if opts[b'switch_parent']:
                 firstparent, secondparent = newparent, newheads[0]
             else:
                 firstparent, secondparent = newheads[0], newparent
-                ui.status(_('updating to %d:%s\n') %
-                          (repo.changelog.rev(firstparent),
-                           short(firstparent)))
+                ui.status(
+                    _(b'updating to %d:%s\n')
+                    % (repo.changelog.rev(firstparent), short(firstparent))
+                )
             hg.clean(repo, firstparent)
-            ui.status(_('merging with %d:%s\n') %
-                      (repo.changelog.rev(secondparent), short(secondparent)))
+            ui.status(
+                _(b'merging with %d:%s\n')
+                % (repo.changelog.rev(secondparent), short(secondparent))
+            )
             err = hg.merge(repo, secondparent, remind=False)
 
         if not err:
             # we don't translate commit messages
-            message = (cmdutil.logmessage(ui, opts) or
-                       ('Automated merge with %s' %
-                        util.removeauth(other.url())))
-            editopt = opts.get('edit') or opts.get('force_editor')
-            editor = cmdutil.getcommiteditor(edit=editopt, editform='fetch')
-            n = repo.commit(message, opts['user'], opts['date'], editor=editor)
-            ui.status(_('new changeset %d:%s merges remote changes '
-                        'with local\n') % (repo.changelog.rev(n),
-                                           short(n)))
+            message = cmdutil.logmessage(ui, opts) or (
+                b'Automated merge with %s' % util.removeauth(other.url())
+            )
+            editopt = opts.get(b'edit') or opts.get(b'force_editor')
+            editor = cmdutil.getcommiteditor(edit=editopt, editform=b'fetch')
+            n = repo.commit(
+                message, opts[b'user'], opts[b'date'], editor=editor
+            )
+            ui.status(
+                _(b'new changeset %d:%s merges remote changes with local\n')
+                % (repo.changelog.rev(n), short(n))
+            )
 
         return err
 
