@@ -21,12 +21,14 @@ from . import (
     branchmap,
     error,
     phases,
+    pycompat,
     scmutil,
     setdiscovery,
     treediscovery,
     util,
 )
 
+
 def findcommonincoming(repo, remote, heads=None, force=False, ancestorsof=None):
     """Return a tuple (common, anyincoming, heads) used to identify the common
     subset of nodes between repo and remote.
@@ -49,20 +51,25 @@ def findcommonincoming(repo, remote, heads=None, force=False, ancestorsof=None):
     extensions a good hook into outgoing.
     """
 
-    if not remote.capable('getbundle'):
+    if not remote.capable(b'getbundle'):
         return treediscovery.findcommonincoming(repo, remote, heads, force)
 
     if heads:
-        knownnode = repo.changelog.hasnode # no nodemap until it is filtered
+        knownnode = repo.changelog.hasnode  # no nodemap until it is filtered
         if all(knownnode(h) for h in heads):
             return (heads, False, heads)
 
-    res = setdiscovery.findcommonheads(repo.ui, repo, remote,
-                                       abortwhenunrelated=not force,
-                                       ancestorsof=ancestorsof)
+    res = setdiscovery.findcommonheads(
+        repo.ui,
+        repo,
+        remote,
+        abortwhenunrelated=not force,
+        ancestorsof=ancestorsof,
+    )
     common, anyinc, srvheads = res
     return (list(common), anyinc, heads or list(srvheads))
 
+
 class outgoing(object):
     '''Represents the set of nodes present in a local repo but not in a
     (possibly) remote one.
@@ -78,8 +85,9 @@ class outgoing(object):
     The sets are computed on demand from the heads, unless provided upfront
     by discovery.'''
 
-    def __init__(self, repo, commonheads=None, missingheads=None,
-                 missingroots=None):
+    def __init__(
+        self, repo, commonheads=None, missingheads=None, missingroots=None
+    ):
         # at least one of them must not be set
         assert None in (commonheads, missingroots)
         cl = repo.changelog
@@ -106,8 +114,9 @@ class outgoing(object):
         self.excluded = []
 
     def _computecommonmissing(self):
-        sets = self._revlog.findcommonmissing(self.commonheads,
-                                              self.missingheads)
+        sets = self._revlog.findcommonmissing(
+            self.commonheads, self.missingheads
+        )
         self._common, self._missing = sets
 
     @util.propertycache
@@ -122,8 +131,10 @@ class outgoing(object):
             self._computecommonmissing()
         return self._missing
 
-def findcommonoutgoing(repo, other, onlyheads=None, force=False,
-                       commoninc=None, portable=False):
+
+def findcommonoutgoing(
+    repo, other, onlyheads=None, force=False, commoninc=None, portable=False
+):
     '''Return an outgoing instance to identify the nodes present in repo but
     not in other.
 
@@ -141,18 +152,19 @@ def findcommonoutgoing(repo, other, onlyheads=None, force=False,
 
     # get common set if not provided
     if commoninc is None:
-        commoninc = findcommonincoming(repo, other, force=force,
-                                       ancestorsof=onlyheads)
+        commoninc = findcommonincoming(
+            repo, other, force=force, ancestorsof=onlyheads
+        )
     og.commonheads, _any, _hds = commoninc
 
     # compute outgoing
-    mayexclude = (repo._phasecache.phaseroots[phases.secret] or repo.obsstore)
+    mayexclude = repo._phasecache.phaseroots[phases.secret] or repo.obsstore
     if not mayexclude:
         og.missingheads = onlyheads or repo.heads()
     elif onlyheads is None:
         # use visible heads as it should be cached
-        og.missingheads = repo.filtered("served").heads()
-        og.excluded = [ctx.node() for ctx in repo.set('secret() or extinct()')]
+        og.missingheads = repo.filtered(b"served").heads()
+        og.excluded = [ctx.node() for ctx in repo.set(b'secret() or extinct()')]
     else:
         # compute common, missing and exclude secret stuff
         sets = repo.changelog.findcommonmissing(og.commonheads, onlyheads)
@@ -167,7 +179,7 @@ def findcommonoutgoing(repo, other, onlyheads=None, force=False,
                 missing.append(node)
         if len(missing) == len(allmissing):
             missingheads = onlyheads
-        else: # update missing heads
+        else:  # update missing heads
             missingheads = phases.newheads(repo, onlyheads, excluded)
         og.missingheads = missingheads
     if portable:
@@ -183,6 +195,7 @@ def findcommonoutgoing(repo, other, onlyheads=None, force=False,
 
     return og
 
+
 def _headssummary(pushop):
     """compute a summary of branch and heads status before and after push
 
@@ -210,11 +223,11 @@ def _headssummary(pushop):
         branches.add(ctx.branch())
 
     with remote.commandexecutor() as e:
-        remotemap = e.callcommand('branchmap', {}).result()
+        remotemap = e.callcommand(b'branchmap', {}).result()
 
-    knownnode = cl.hasnode # do not use nodemap until it is filtered
+    knownnode = cl.hasnode  # do not use nodemap until it is filtered
     # A. register remote heads of branches which are in outgoing set
-    for branch, heads in remotemap.iteritems():
+    for branch, heads in pycompat.iteritems(remotemap):
         # don't add head info about branches which we don't have locally
         if branch not in branches:
             continue
@@ -234,13 +247,15 @@ def _headssummary(pushop):
 
     # C. Update newmap with outgoing changes.
     # This will possibly add new heads and remove existing ones.
-    newmap = branchmap.remotebranchcache((branch, heads[1])
-                                 for branch, heads in headssum.iteritems()
-                                 if heads[0] is not None)
+    newmap = branchmap.remotebranchcache(
+        (branch, heads[1])
+        for branch, heads in pycompat.iteritems(headssum)
+        if heads[0] is not None
+    )
     newmap.update(repo, (ctx.rev() for ctx in missingctx))
-    for branch, newheads in newmap.iteritems():
+    for branch, newheads in pycompat.iteritems(newmap):
         headssum[branch][1][:] = newheads
-    for branch, items in headssum.iteritems():
+    for branch, items in pycompat.iteritems(headssum):
         for l in items:
             if l is not None:
                 l.sort()
@@ -252,27 +267,32 @@ def _headssummary(pushop):
         futureheads = set(torev(h) for h in outgoing.missingheads)
         futureheads |= set(torev(h) for h in outgoing.commonheads)
         allfuturecommon = repo.changelog.ancestors(futureheads, inclusive=True)
-        for branch, heads in sorted(headssum.iteritems()):
+        for branch, heads in sorted(pycompat.iteritems(headssum)):
             remoteheads, newheads, unsyncedheads, placeholder = heads
             result = _postprocessobsolete(pushop, allfuturecommon, newheads)
-            headssum[branch] = (remoteheads, sorted(result[0]), unsyncedheads,
-                                sorted(result[1]))
+            headssum[branch] = (
+                remoteheads,
+                sorted(result[0]),
+                unsyncedheads,
+                sorted(result[1]),
+            )
     return headssum
 
+
 def _oldheadssummary(repo, remoteheads, outgoing, inc=False):
     """Compute branchmapsummary for repo without branchmap support"""
 
     # 1-4b. old servers: Check for new topological heads.
     # Construct {old,new}map with branch = None (topological branch).
     # (code based on update)
-    knownnode = repo.changelog.hasnode # no nodemap until it is filtered
+    knownnode = repo.changelog.hasnode  # no nodemap until it is filtered
     oldheads = sorted(h for h in remoteheads if knownnode(h))
     # all nodes in outgoing.missing are children of either:
     # - an element of oldheads
     # - another element of outgoing.missing
     # - nullrev
     # This explains why the new head are very simple to compute.
-    r = repo.set('heads(%ln + %ln)', oldheads, outgoing.missing)
+    r = repo.set(b'heads(%ln + %ln)', oldheads, outgoing.missing)
     newheads = sorted(c.node() for c in r)
     # set some unsynced head to issue the "unsynced changes" warning
     if inc:
@@ -281,6 +301,7 @@ def _oldheadssummary(repo, remoteheads, outgoing, inc=False):
         unsynced = []
     return {None: (oldheads, newheads, unsynced, [])}
 
+
 def _nowarnheads(pushop):
     # Compute newly pushed bookmarks. We don't warn about bookmarked heads.
     repo = pushop.repo.unfiltered()
@@ -288,20 +309,22 @@ def _nowarnheads(pushop):
     localbookmarks = repo._bookmarks
 
     with remote.commandexecutor() as e:
-        remotebookmarks = e.callcommand('listkeys', {
-            'namespace': 'bookmarks',
-        }).result()
+        remotebookmarks = e.callcommand(
+            b'listkeys', {b'namespace': b'bookmarks',}
+        ).result()
 
     bookmarkedheads = set()
 
     # internal config: bookmarks.pushing
-    newbookmarks = [localbookmarks.expandname(b)
-                    for b in pushop.ui.configlist('bookmarks', 'pushing')]
+    newbookmarks = [
+        localbookmarks.expandname(b)
+        for b in pushop.ui.configlist(b'bookmarks', b'pushing')
+    ]
 
     for bm in localbookmarks:
         rnode = remotebookmarks.get(bm)
         if rnode and rnode in repo:
-            lctx, rctx = localbookmarks.changectx(bm), repo[rnode]
+            lctx, rctx = repo[localbookmarks[bm]], repo[rnode]
             if bookmarks.validdest(repo, rctx, lctx):
                 bookmarkedheads.add(lctx.node())
         else:
@@ -310,6 +333,7 @@ def _nowarnheads(pushop):
 
     return bookmarkedheads
 
+
 def checkheads(pushop):
     """Check that a push won't add any outgoing head
 
@@ -333,20 +357,34 @@ def checkheads(pushop):
         # remote is empty, nothing to check.
         return
 
-    if remote.capable('branchmap'):
+    if remote.capable(b'branchmap'):
         headssum = _headssummary(pushop)
     else:
         headssum = _oldheadssummary(repo, remoteheads, outgoing, inc)
     pushop.pushbranchmap = headssum
-    newbranches = [branch for branch, heads in headssum.iteritems()
-                   if heads[0] is None]
+    newbranches = [
+        branch
+        for branch, heads in pycompat.iteritems(headssum)
+        if heads[0] is None
+    ]
     # 1. Check for new branches on the remote.
     if newbranches and not newbranch:  # new branch requires --new-branch
-        branchnames = ', '.join(sorted(newbranches))
-        raise error.Abort(_("push creates new remote branches: %s!")
-                           % branchnames,
-                         hint=_("use 'hg push --new-branch' to create"
-                                " new remote branches"))
+        branchnames = b', '.join(sorted(newbranches))
+        # Calculate how many of the new branches are closed branches
+        closedbranches = set()
+        for tag, heads, tip, isclosed in repo.branchmap().iterbranches():
+            if isclosed:
+                closedbranches.add(tag)
+        closedbranches = closedbranches & set(newbranches)
+        if closedbranches:
+            errmsg = _(b"push creates new remote branches: %s (%d closed)!") % (
+                branchnames,
+                len(closedbranches),
+            )
+        else:
+            errmsg = _(b"push creates new remote branches: %s!") % branchnames
+        hint = _(b"use 'hg push --new-branch' to create new remote branches")
+        raise error.Abort(errmsg, hint=hint)
 
     # 2. Find heads that we need not warn about
     nowarnheads = _nowarnheads(pushop)
@@ -355,7 +393,7 @@ def checkheads(pushop):
     # If there are more heads after the push than before, a suitable
     # error message, depending on unsynced status, is displayed.
     errormsg = None
-    for branch, heads in sorted(headssum.iteritems()):
+    for branch, heads in sorted(pycompat.iteritems(headssum)):
         remoteheads, newheads, unsyncedheads, discardedheads = heads
         # add unsynced data
         if remoteheads is None:
@@ -363,7 +401,7 @@ def checkheads(pushop):
         else:
             oldhs = set(remoteheads)
         oldhs.update(unsyncedheads)
-        dhs = None # delta heads, the new heads on branch
+        dhs = None  # delta heads, the new heads on branch
         newhs = set(newheads)
         newhs.update(unsyncedheads)
         if unsyncedheads:
@@ -373,55 +411,75 @@ def checkheads(pushop):
             else:
                 heads = scmutil.nodesummaries(repo, unsyncedheads)
             if heads is None:
-                repo.ui.status(_("remote has heads that are "
-                                 "not known locally\n"))
+                repo.ui.status(
+                    _(b"remote has heads that are not known locally\n")
+                )
             elif branch is None:
-                repo.ui.status(_("remote has heads that are "
-                                 "not known locally: %s\n") % heads)
+                repo.ui.status(
+                    _(b"remote has heads that are not known locally: %s\n")
+                    % heads
+                )
             else:
-                repo.ui.status(_("remote has heads on branch '%s' that are "
-                                 "not known locally: %s\n") % (branch, heads))
+                repo.ui.status(
+                    _(
+                        b"remote has heads on branch '%s' that are "
+                        b"not known locally: %s\n"
+                    )
+                    % (branch, heads)
+                )
         if remoteheads is None:
             if len(newhs) > 1:
                 dhs = list(newhs)
                 if errormsg is None:
-                    errormsg = (_("push creates new branch '%s' "
-                                  "with multiple heads") % (branch))
-                    hint = _("merge or"
-                             " see 'hg help push' for details about"
-                             " pushing new heads")
+                    errormsg = (
+                        _(b"push creates new branch '%s' with multiple heads")
+                        % branch
+                    )
+                    hint = _(
+                        b"merge or"
+                        b" see 'hg help push' for details about"
+                        b" pushing new heads"
+                    )
         elif len(newhs) > len(oldhs):
             # remove bookmarked or existing remote heads from the new heads list
             dhs = sorted(newhs - nowarnheads - oldhs)
         if dhs:
             if errormsg is None:
-                if branch not in ('default', None):
-                    errormsg = _("push creates new remote head %s "
-                                 "on branch '%s'!") % (short(dhs[0]), branch)
+                if branch not in (b'default', None):
+                    errormsg = _(
+                        b"push creates new remote head %s on branch '%s'!"
+                    ) % (short(dhs[0]), branch)
                 elif repo[dhs[0]].bookmarks():
-                    errormsg = _("push creates new remote head %s "
-                                 "with bookmark '%s'!") % (
-                                 short(dhs[0]), repo[dhs[0]].bookmarks()[0])
+                    errormsg = _(
+                        b"push creates new remote head %s "
+                        b"with bookmark '%s'!"
+                    ) % (short(dhs[0]), repo[dhs[0]].bookmarks()[0])
                 else:
-                    errormsg = _("push creates new remote head %s!"
-                                 ) % short(dhs[0])
+                    errormsg = _(b"push creates new remote head %s!") % short(
+                        dhs[0]
+                    )
                 if unsyncedheads:
-                    hint = _("pull and merge or"
-                             " see 'hg help push' for details about"
-                             " pushing new heads")
+                    hint = _(
+                        b"pull and merge or"
+                        b" see 'hg help push' for details about"
+                        b" pushing new heads"
+                    )
                 else:
-                    hint = _("merge or"
-                             " see 'hg help push' for details about"
-                             " pushing new heads")
+                    hint = _(
+                        b"merge or"
+                        b" see 'hg help push' for details about"
+                        b" pushing new heads"
+                    )
             if branch is None:
-                repo.ui.note(_("new remote heads:\n"))
+                repo.ui.note(_(b"new remote heads:\n"))
             else:
-                repo.ui.note(_("new remote heads on branch '%s':\n") % branch)
+                repo.ui.note(_(b"new remote heads on branch '%s':\n") % branch)
             for h in dhs:
-                repo.ui.note((" %s\n") % short(h))
+                repo.ui.note(b" %s\n" % short(h))
     if errormsg:
         raise error.Abort(errormsg, hint=hint)
 
+
 def _postprocessobsolete(pushop, futurecommon, candidate_newhs):
     """post process the list of new heads with obsolescence information
 
@@ -440,26 +498,27 @@ def _postprocessobsolete(pushop, futurecommon, candidate_newhs):
     # define various utilities and containers
     repo = pushop.repo
     unfi = repo.unfiltered()
-    tonode = unfi.changelog.node
-    torev = unfi.changelog.nodemap.get
+    torev = unfi.changelog.index.get_rev
     public = phases.public
     getphase = unfi._phasecache.phase
-    ispublic = (lambda r: getphase(unfi, r) == public)
-    ispushed = (lambda n: torev(n) in futurecommon)
+    ispublic = lambda r: getphase(unfi, r) == public
+    ispushed = lambda n: torev(n) in futurecommon
     hasoutmarker = functools.partial(pushingmarkerfor, unfi.obsstore, ispushed)
     successorsmarkers = unfi.obsstore.successors
-    newhs = set() # final set of new heads
-    discarded = set() # new head of fully replaced branch
+    newhs = set()  # final set of new heads
+    discarded = set()  # new head of fully replaced branch
 
-    localcandidate = set() # candidate heads known locally
-    unknownheads = set() # candidate heads unknown locally
+    localcandidate = set()  # candidate heads known locally
+    unknownheads = set()  # candidate heads unknown locally
     for h in candidate_newhs:
         if h in unfi:
             localcandidate.add(h)
         else:
             if successorsmarkers.get(h) is not None:
-                msg = ('checkheads: remote head unknown locally has'
-                       ' local marker: %s\n')
+                msg = (
+                    b'checkheads: remote head unknown locally has'
+                    b' local marker: %s\n'
+                )
                 repo.ui.debug(msg % hex(h))
             unknownheads.add(h)
 
@@ -470,24 +529,33 @@ def _postprocessobsolete(pushop, futurecommon, candidate_newhs):
     # actually process branch replacement
     while localcandidate:
         nh = localcandidate.pop()
+        current_branch = unfi[nh].branch()
         # run this check early to skip the evaluation of the whole branch
-        if (torev(nh) in futurecommon or ispublic(torev(nh))):
+        if torev(nh) in futurecommon or ispublic(torev(nh)):
             newhs.add(nh)
             continue
 
         # Get all revs/nodes on the branch exclusive to this head
         # (already filtered heads are "ignored"))
-        branchrevs = unfi.revs('only(%n, (%ln+%ln))',
-                               nh, localcandidate, newhs)
-        branchnodes = [tonode(r) for r in branchrevs]
+        branchrevs = unfi.revs(
+            b'only(%n, (%ln+%ln))', nh, localcandidate, newhs
+        )
+
+        branchnodes = []
+        for r in branchrevs:
+            c = unfi[r]
+            if c.branch() == current_branch:
+                branchnodes.append(c.node())
 
         # The branch won't be hidden on the remote if
         # * any part of it is public,
         # * any part of it is considered part of the result by previous logic,
         # * if we have no markers to push to obsolete it.
-        if (any(ispublic(r) for r in branchrevs)
-                or any(torev(n) in futurecommon for n in branchnodes)
-                or any(not hasoutmarker(n) for n in branchnodes)):
+        if (
+            any(ispublic(r) for r in branchrevs)
+            or any(torev(n) in futurecommon for n in branchnodes)
+            or any(not hasoutmarker(n) for n in branchnodes)
+        ):
             newhs.add(nh)
         else:
             # note: there is a corner case if there is a merge in the branch.
@@ -498,6 +566,7 @@ def _postprocessobsolete(pushop, futurecommon, candidate_newhs):
     newhs |= unknownheads
     return newhs, discarded
 
+
 def pushingmarkerfor(obsstore, ispushed, node):
     """true if some markers are to be pushed for node
 
@@ -519,9 +588,9 @@ def pushingmarkerfor(obsstore, ispushed, node):
         markers = successorsmarkers.get(current, ())
         # markers fields = ('prec', 'succs', 'flag', 'meta', 'date', 'parents')
         for m in markers:
-            nexts = m[1] # successors
-            if not nexts: # this is a prune marker
-                nexts = m[5] or () # parents
+            nexts = m[1]  # successors
+            if not nexts:  # this is a prune marker
+                nexts = m[5] or ()  # parents
             for n in nexts:
                 if n not in seen:
                     seen.add(n)
