@@ -112,6 +112,11 @@ from .node import (
     nullid,
     nullrev,
     short,
+    wdirrev,
+)
+from .pycompat import (
+    getattr,
+    setattr,
 )
 from . import (
     error,
@@ -121,31 +126,33 @@ from . import (
     util,
 )
 
-_fphasesentry = struct.Struct('>i20s')
+_fphasesentry = struct.Struct(b'>i20s')
 
-INTERNAL_FLAG = 64 # Phases for mercurial internal usage only
-HIDEABLE_FLAG = 32 # Phases that are hideable
+INTERNAL_FLAG = 64  # Phases for mercurial internal usage only
+HIDEABLE_FLAG = 32  # Phases that are hideable
 
 # record phase index
 public, draft, secret = range(3)
 internal = INTERNAL_FLAG | HIDEABLE_FLAG
 archived = HIDEABLE_FLAG
-allphases = range(internal + 1)
+allphases = list(range(internal + 1))
 trackedphases = allphases[1:]
 # record phase names
-cmdphasenames = ['public', 'draft', 'secret']  # known to `hg phase` command
+cmdphasenames = [b'public', b'draft', b'secret']  # known to `hg phase` command
 phasenames = [None] * len(allphases)
-phasenames[:len(cmdphasenames)] = cmdphasenames
-phasenames[archived] = 'archived'
-phasenames[internal] = 'internal'
+phasenames[: len(cmdphasenames)] = cmdphasenames
+phasenames[archived] = b'archived'
+phasenames[internal] = b'internal'
 # record phase property
 mutablephases = tuple(allphases[1:])
 remotehiddenphases = tuple(allphases[2:])
 localhiddenphases = tuple(p for p in allphases if p & HIDEABLE_FLAG)
 
+
 def supportinternal(repo):
     """True if the internal phase can be used on a repository"""
-    return 'internal-phase' in repo.requirements
+    return b'internal-phase' in repo.requirements
+
 
 def _readroots(repo, phasedefaults=None):
     """Read phase roots from disk
@@ -162,7 +169,7 @@ def _readroots(repo, phasedefaults=None):
     dirty = False
     roots = [set() for i in allphases]
     try:
-        f, pending = txnutil.trypending(repo.root, repo.svfs, 'phaseroots')
+        f, pending = txnutil.trypending(repo.root, repo.svfs, b'phaseroots')
         try:
             for line in f:
                 phase, nh = line.split()
@@ -178,6 +185,7 @@ def _readroots(repo, phasedefaults=None):
         dirty = True
     return roots, dirty
 
+
 def binaryencode(phasemapping):
     """encode a 'phase -> nodes' mapping into a binary stream
 
@@ -188,7 +196,8 @@ def binaryencode(phasemapping):
     for phase, nodes in enumerate(phasemapping):
         for head in nodes:
             binarydata.append(_fphasesentry.pack(phase, head))
-    return ''.join(binarydata)
+    return b''.join(binarydata)
+
 
 def binarydecode(stream):
     """decode a binary stream into a 'phase -> nodes' mapping
@@ -200,12 +209,13 @@ def binarydecode(stream):
         entry = stream.read(entrysize)
         if len(entry) < entrysize:
             if entry:
-                raise error.Abort(_('bad phase-heads stream'))
+                raise error.Abort(_(b'bad phase-heads stream'))
             break
         phase, node = _fphasesentry.unpack(entry)
         headsbyphase[phase].append(node)
     return headsbyphase
 
+
 def _trackphasechange(data, rev, old, new):
     """add a phase move the <data> dictionnary
 
@@ -218,6 +228,7 @@ def _trackphasechange(data, rev, old, new):
         old = existing[0]
     data[rev] = (old, new)
 
+
 class phasecache(object):
     def __init__(self, repo, phasedefaults, _load=True):
         if _load:
@@ -230,33 +241,53 @@ class phasecache(object):
 
     def getrevset(self, repo, phases, subset=None):
         """return a smartset for the given phases"""
-        self.loadphaserevs(repo) # ensure phase's sets are loaded
+        self.loadphaserevs(repo)  # ensure phase's sets are loaded
         phases = set(phases)
-        if public not in phases:
-            # fast path: _phasesets contains the interesting sets,
-            # might only need a union and post-filtering.
-            if len(phases) == 1:
-                [p] = phases
-                revs = self._phasesets[p]
-            else:
-                revs = set.union(*[self._phasesets[p] for p in phases])
-            if repo.changelog.filteredrevs:
-                revs = revs - repo.changelog.filteredrevs
-            if subset is None:
-                return smartset.baseset(revs)
-            else:
-                return subset & smartset.baseset(revs)
-        else:
+        publicphase = public in phases
+
+        if publicphase:
+            # In this case, phases keeps all the *other* phases.
             phases = set(allphases).difference(phases)
             if not phases:
                 return smartset.fullreposet(repo)
-            if len(phases) == 1:
-                [p] = phases
-                revs = self._phasesets[p]
+
+        # fast path: _phasesets contains the interesting sets,
+        # might only need a union and post-filtering.
+        revsneedscopy = False
+        if len(phases) == 1:
+            [p] = phases
+            revs = self._phasesets[p]
+            revsneedscopy = True  # Don't modify _phasesets
+        else:
+            # revs has the revisions in all *other* phases.
+            revs = set.union(*[self._phasesets[p] for p in phases])
+
+        def _addwdir(wdirsubset, wdirrevs):
+            if wdirrev in wdirsubset and repo[None].phase() in phases:
+                if revsneedscopy:
+                    wdirrevs = wdirrevs.copy()
+                # The working dir would never be in the # cache, but it was in
+                # the subset being filtered for its phase (or filtered out,
+                # depending on publicphase), so add it to the output to be
+                # included (or filtered out).
+                wdirrevs.add(wdirrev)
+            return wdirrevs
+
+        if not publicphase:
+            if repo.changelog.filteredrevs:
+                revs = revs - repo.changelog.filteredrevs
+
+            if subset is None:
+                return smartset.baseset(revs)
             else:
-                revs = set.union(*[self._phasesets[p] for p in phases])
+                revs = _addwdir(subset, revs)
+                return subset & smartset.baseset(revs)
+        else:
             if subset is None:
                 subset = smartset.fullreposet(repo)
+
+            revs = _addwdir(subset, revs)
+
             if not revs:
                 return subset
             return subset.filter(lambda r: r not in revs)
@@ -274,16 +305,22 @@ class phasecache(object):
 
     def replace(self, phcache):
         """replace all values in 'self' with content of phcache"""
-        for a in ('phaseroots', 'dirty', 'opener', '_loadedrevslen',
-                  '_phasesets'):
+        for a in (
+            b'phaseroots',
+            b'dirty',
+            b'opener',
+            b'_loadedrevslen',
+            b'_phasesets',
+        ):
             setattr(self, a, getattr(phcache, a))
 
     def _getphaserevsnative(self, repo):
         repo = repo.unfiltered()
         nativeroots = []
         for phase in trackedphases:
-            nativeroots.append(pycompat.maplist(repo.changelog.rev,
-                                                self.phaseroots[phase]))
+            nativeroots.append(
+                pycompat.maplist(repo.changelog.rev, self.phaseroots[phase])
+            )
         return repo.changelog.computephases(nativeroots)
 
     def _computephaserevspure(self, repo):
@@ -324,7 +361,7 @@ class phasecache(object):
         if rev == nullrev:
             return public
         if rev < nullrev:
-            raise ValueError(_('cannot lookup negative revision'))
+            raise ValueError(_(b'cannot lookup negative revision'))
         if rev >= self._loadedrevslen:
             self.invalidate()
             self.loadphaserevs(repo)
@@ -336,7 +373,7 @@ class phasecache(object):
     def write(self):
         if not self.dirty:
             return
-        f = self.opener('phaseroots', 'w', atomictemp=True, checkambig=True)
+        f = self.opener(b'phaseroots', b'w', atomictemp=True, checkambig=True)
         try:
             self._write(f)
         finally:
@@ -345,7 +382,7 @@ class phasecache(object):
     def _write(self, fp):
         for phase, roots in enumerate(self.phaseroots):
             for h in sorted(roots):
-                fp.write('%i %s\n' % (phase, hex(h)))
+                fp.write(b'%i %s\n' % (phase, hex(h)))
         self.dirty = False
 
     def _updateroots(self, phase, newroots, tr):
@@ -353,14 +390,14 @@ class phasecache(object):
         self.invalidate()
         self.dirty = True
 
-        tr.addfilegenerator('phase', ('phaseroots',), self._write)
-        tr.hookargs['phases_moved'] = '1'
+        tr.addfilegenerator(b'phase', (b'phaseroots',), self._write)
+        tr.hookargs[b'phases_moved'] = b'1'
 
     def registernew(self, repo, tr, targetphase, nodes):
         repo = repo.unfiltered()
         self._retractboundary(repo, tr, targetphase, nodes)
-        if tr is not None and 'phases' in tr.changes:
-            phasetracking = tr.changes['phases']
+        if tr is not None and b'phases' in tr.changes:
+            phasetracking = tr.changes[b'phases']
             torev = repo.changelog.rev
             phase = self.phase
             for n in nodes:
@@ -383,31 +420,35 @@ class phasecache(object):
         if tr is None:
             phasetracking = None
         else:
-            phasetracking = tr.changes.get('phases')
+            phasetracking = tr.changes.get(b'phases')
 
         repo = repo.unfiltered()
 
-        changes = set() # set of revisions to be changed
-        delroots = [] # set of root deleted by this path
+        changes = set()  # set of revisions to be changed
+        delroots = []  # set of root deleted by this path
         for phase in pycompat.xrange(targetphase + 1, len(allphases)):
             # filter nodes that are not in a compatible phase already
-            nodes = [n for n in nodes
-                     if self.phase(repo, repo[n].rev()) >= phase]
+            nodes = [
+                n for n in nodes if self.phase(repo, repo[n].rev()) >= phase
+            ]
             if not nodes:
-                break # no roots to move anymore
+                break  # no roots to move anymore
 
             olds = self.phaseroots[phase]
 
-            affected = repo.revs('%ln::%ln', olds, nodes)
+            affected = repo.revs(b'%ln::%ln', olds, nodes)
             changes.update(affected)
             if dryrun:
                 continue
             for r in affected:
-                _trackphasechange(phasetracking, r, self.phase(repo, r),
-                                  targetphase)
+                _trackphasechange(
+                    phasetracking, r, self.phase(repo, r), targetphase
+                )
 
-            roots = set(ctx.node() for ctx in repo.set(
-                    'roots((%ln::) - %ld)', olds, affected))
+            roots = set(
+                ctx.node()
+                for ctx in repo.set(b'roots((%ln::) - %ld)', olds, affected)
+            )
             if olds != roots:
                 self._updateroots(phase, roots, tr)
                 # some roots may need to be declared for lower phases
@@ -420,27 +461,29 @@ class phasecache(object):
         return changes
 
     def retractboundary(self, repo, tr, targetphase, nodes):
-        oldroots = self.phaseroots[:targetphase + 1]
+        oldroots = self.phaseroots[: targetphase + 1]
         if tr is None:
             phasetracking = None
         else:
-            phasetracking = tr.changes.get('phases')
+            phasetracking = tr.changes.get(b'phases')
         repo = repo.unfiltered()
-        if (self._retractboundary(repo, tr, targetphase, nodes)
-            and phasetracking is not None):
+        if (
+            self._retractboundary(repo, tr, targetphase, nodes)
+            and phasetracking is not None
+        ):
 
             # find the affected revisions
             new = self.phaseroots[targetphase]
             old = oldroots[targetphase]
-            affected = set(repo.revs('(%ln::) - (%ln::)', new, old))
+            affected = set(repo.revs(b'(%ln::) - (%ln::)', new, old))
 
             # find the phase of the affected revision
             for phase in pycompat.xrange(targetphase, -1, -1):
                 if phase:
                     roots = oldroots[phase]
-                    revs = set(repo.revs('%ln::%ld', roots, affected))
+                    revs = set(repo.revs(b'%ln::%ld', roots, affected))
                     affected -= revs
-                else: # public phase
+                else:  # public phase
                     revs = affected
                 for r in revs:
                     _trackphasechange(phasetracking, r, phase, targetphase)
@@ -451,30 +494,33 @@ class phasecache(object):
         # phaseroots values, replace them.
         if targetphase in (archived, internal) and not supportinternal(repo):
             name = phasenames[targetphase]
-            msg = 'this repository does not support the %s phase' % name
+            msg = b'this repository does not support the %s phase' % name
             raise error.ProgrammingError(msg)
 
         repo = repo.unfiltered()
         currentroots = self.phaseroots[targetphase]
         finalroots = oldroots = set(currentroots)
-        newroots = [n for n in nodes
-                    if self.phase(repo, repo[n].rev()) < targetphase]
+        newroots = [
+            n for n in nodes if self.phase(repo, repo[n].rev()) < targetphase
+        ]
         if newroots:
 
             if nullid in newroots:
-                raise error.Abort(_('cannot change null revision phase'))
+                raise error.Abort(_(b'cannot change null revision phase'))
             currentroots = currentroots.copy()
             currentroots.update(newroots)
 
             # Only compute new roots for revs above the roots that are being
             # retracted.
             minnewroot = min(repo[n].rev() for n in newroots)
-            aboveroots = [n for n in currentroots
-                          if repo[n].rev() >= minnewroot]
-            updatedroots = repo.set('roots(%ln::)', aboveroots)
+            aboveroots = [
+                n for n in currentroots if repo[n].rev() >= minnewroot
+            ]
+            updatedroots = repo.set(b'roots(%ln::)', aboveroots)
 
-            finalroots = set(n for n in currentroots if repo[n].rev() <
-                             minnewroot)
+            finalroots = set(
+                n for n in currentroots if repo[n].rev() < minnewroot
+            )
             finalroots.update(ctx.node() for ctx in updatedroots)
         if finalroots != oldroots:
             self._updateroots(targetphase, finalroots, tr)
@@ -487,14 +533,15 @@ class phasecache(object):
         Nothing is lost as unknown nodes only hold data for their descendants.
         """
         filtered = False
-        nodemap = repo.changelog.nodemap # to filter unknown nodes
+        has_node = repo.changelog.index.has_node  # to filter unknown nodes
         for phase, nodes in enumerate(self.phaseroots):
-            missing = sorted(node for node in nodes if node not in nodemap)
+            missing = sorted(node for node in nodes if not has_node(node))
             if missing:
                 for mnode in missing:
                     repo.ui.debug(
-                        'removing unknown node %s from %i-phase boundary\n'
-                        % (short(mnode), phase))
+                        b'removing unknown node %s from %i-phase boundary\n'
+                        % (short(mnode), phase)
+                    )
                 nodes.symmetric_difference_update(missing)
                 filtered = True
         if filtered:
@@ -509,6 +556,7 @@ class phasecache(object):
         # (see branchmap one)
         self.invalidate()
 
+
 def advanceboundary(repo, tr, targetphase, nodes, dryrun=None):
     """Add nodes to a phase changing other nodes phases if necessary.
 
@@ -522,12 +570,14 @@ def advanceboundary(repo, tr, targetphase, nodes, dryrun=None):
     Returns a set of revs whose phase is changed or should be changed
     """
     phcache = repo._phasecache.copy()
-    changes = phcache.advanceboundary(repo, tr, targetphase, nodes,
-                                      dryrun=dryrun)
+    changes = phcache.advanceboundary(
+        repo, tr, targetphase, nodes, dryrun=dryrun
+    )
     if not dryrun:
         repo._phasecache.replace(phcache)
     return changes
 
+
 def retractboundary(repo, tr, targetphase, nodes):
     """Set nodes back to a phase changing other nodes phases if
     necessary.
@@ -540,6 +590,7 @@ def retractboundary(repo, tr, targetphase, nodes):
     phcache.retractboundary(repo, tr, targetphase, nodes)
     repo._phasecache.replace(phcache)
 
+
 def registernew(repo, tr, targetphase, nodes):
     """register a new revision and its phase
 
@@ -550,11 +601,12 @@ def registernew(repo, tr, targetphase, nodes):
     phcache.registernew(repo, tr, targetphase, nodes)
     repo._phasecache.replace(phcache)
 
+
 def listphases(repo):
     """List phases root for serialization over pushkey"""
     # Use ordered dictionary so behavior is deterministic.
     keys = util.sortdict()
-    value = '%i' % draft
+    value = b'%i' % draft
     cl = repo.unfiltered().changelog
     for root in repo._phasecache.phaseroots[draft]:
         if repo._phasecache.phase(repo, cl.rev(root)) <= draft:
@@ -577,18 +629,19 @@ def listphases(repo):
         #
         # The server can't handle it on it's own as it has no idea of
         # client phase data.
-        keys['publishing'] = 'True'
+        keys[b'publishing'] = b'True'
     return keys
 
+
 def pushphase(repo, nhex, oldphasestr, newphasestr):
     """List phases root for serialization over pushkey"""
     repo = repo.unfiltered()
     with repo.lock():
         currentphase = repo[nhex].phase()
-        newphase = abs(int(newphasestr)) # let's avoid negative index surprise
-        oldphase = abs(int(oldphasestr)) # let's avoid negative index surprise
+        newphase = abs(int(newphasestr))  # let's avoid negative index surprise
+        oldphase = abs(int(oldphasestr))  # let's avoid negative index surprise
         if currentphase == oldphase and newphase < oldphase:
-            with repo.transaction('pushkey-phase') as tr:
+            with repo.transaction(b'pushkey-phase') as tr:
                 advanceboundary(repo, tr, newphase, [bin(nhex)])
             return True
         elif currentphase == newphase:
@@ -597,6 +650,7 @@ def pushphase(repo, nhex, oldphasestr, newphasestr):
         else:
             return False
 
+
 def subsetphaseheads(repo, subset):
     """Finds the phase heads for a subset of a history
 
@@ -609,10 +663,11 @@ def subsetphaseheads(repo, subset):
     # No need to keep track of secret phase; any heads in the subset that
     # are not mentioned are implicitly secret.
     for phase in allphases[:secret]:
-        revset = "heads(%%ln & %s())" % phasenames[phase]
+        revset = b"heads(%%ln & %s())" % phasenames[phase]
         headsbyphase[phase] = [cl.node(r) for r in repo.revs(revset, subset)]
     return headsbyphase
 
+
 def updatephases(repo, trgetter, headsbyphase):
     """Updates the repo with the given phase heads"""
     # Now advance phase boundaries of all but secret phase
@@ -621,11 +676,12 @@ def updatephases(repo, trgetter, headsbyphase):
     # to update. This avoid creating empty transaction during no-op operation.
 
     for phase in allphases[:-1]:
-        revset = '%ln - _phase(%s)'
+        revset = b'%ln - _phase(%s)'
         heads = [c.node() for c in repo.set(revset, headsbyphase[phase], phase)]
         if heads:
             advanceboundary(repo, trgetter(), phase, heads)
 
+
 def analyzeremotephases(repo, subset, roots):
     """Compute phases heads and root in a subset of node from root dict
 
@@ -637,26 +693,34 @@ def analyzeremotephases(repo, subset, roots):
     repo = repo.unfiltered()
     # build list from dictionary
     draftroots = []
-    nodemap = repo.changelog.nodemap # to filter unknown nodes
-    for nhex, phase in roots.iteritems():
-        if nhex == 'publishing': # ignore data related to publish option
+    has_node = repo.changelog.index.has_node  # to filter unknown nodes
+    for nhex, phase in pycompat.iteritems(roots):
+        if nhex == b'publishing':  # ignore data related to publish option
             continue
         node = bin(nhex)
         phase = int(phase)
         if phase == public:
             if node != nullid:
-                repo.ui.warn(_('ignoring inconsistent public root'
-                               ' from remote: %s\n') % nhex)
+                repo.ui.warn(
+                    _(
+                        b'ignoring inconsistent public root'
+                        b' from remote: %s\n'
+                    )
+                    % nhex
+                )
         elif phase == draft:
-            if node in nodemap:
+            if has_node(node):
                 draftroots.append(node)
         else:
-            repo.ui.warn(_('ignoring unexpected root from remote: %i %s\n')
-                         % (phase, nhex))
+            repo.ui.warn(
+                _(b'ignoring unexpected root from remote: %i %s\n')
+                % (phase, nhex)
+            )
     # compute heads
     publicheads = newheads(repo, subset, draftroots)
     return publicheads, draftroots
 
+
 class remotephasessummary(object):
     """summarize phase information on the remote side
 
@@ -670,14 +734,15 @@ class remotephasessummary(object):
         unfi = repo.unfiltered()
         self._allremoteroots = remoteroots
 
-        self.publishing = remoteroots.get('publishing', False)
+        self.publishing = remoteroots.get(b'publishing', False)
 
         ana = analyzeremotephases(repo, remotesubset, remoteroots)
         self.publicheads, self.draftroots = ana
         # Get the list of all "heads" revs draft on remote
-        dheads = unfi.set('heads(%ln::%ln)', self.draftroots, remotesubset)
+        dheads = unfi.set(b'heads(%ln::%ln)', self.draftroots, remotesubset)
         self.draftheads = [c.node() for c in dheads]
 
+
 def newheads(repo, heads, roots):
     """compute new head of a subset minus another
 
@@ -689,7 +754,7 @@ def newheads(repo, heads, roots):
 
     repo = repo.unfiltered()
     cl = repo.changelog
-    rev = cl.nodemap.get
+    rev = cl.index.get_rev
     if not roots:
         return heads
     if not heads or heads == [nullid]:
@@ -698,48 +763,50 @@ def newheads(repo, heads, roots):
     new_heads = set(rev(n) for n in heads if n != nullid)
     roots = [rev(n) for n in roots]
     # compute the area we need to remove
-    affected_zone = repo.revs("(%ld::%ld)", roots, new_heads)
+    affected_zone = repo.revs(b"(%ld::%ld)", roots, new_heads)
     # heads in the area are no longer heads
     new_heads.difference_update(affected_zone)
     # revisions in the area have children outside of it,
     # They might be new heads
-    candidates = repo.revs("parents(%ld + (%ld and merge())) and not null",
-                           roots, affected_zone)
+    candidates = repo.revs(
+        b"parents(%ld + (%ld and merge())) and not null", roots, affected_zone
+    )
     candidates -= affected_zone
     if new_heads or candidates:
         # remove candidate that are ancestors of other heads
         new_heads.update(candidates)
-        prunestart = repo.revs("parents(%ld) and not null", new_heads)
+        prunestart = repo.revs(b"parents(%ld) and not null", new_heads)
         pruned = dagop.reachableroots(repo, candidates, prunestart)
         new_heads.difference_update(pruned)
 
     return pycompat.maplist(cl.node, sorted(new_heads))
 
+
 def newcommitphase(ui):
     """helper to get the target phase of new commit
 
     Handle all possible values for the phases.new-commit options.
 
     """
-    v = ui.config('phases', 'new-commit')
+    v = ui.config(b'phases', b'new-commit')
     try:
         return phasenames.index(v)
     except ValueError:
         try:
             return int(v)
         except ValueError:
-            msg = _("phases.new-commit: not a valid phase name ('%s')")
+            msg = _(b"phases.new-commit: not a valid phase name ('%s')")
             raise error.ConfigError(msg % v)
 
+
 def hassecret(repo):
     """utility function that check if a repo have any secret changeset."""
     return bool(repo._phasecache.phaseroots[2])
 
+
 def preparehookargs(node, old, new):
     if old is None:
-        old = ''
+        old = b''
     else:
         old = phasenames[old]
-    return {'node': node,
-            'oldphase': old,
-            'phase': phasenames[new]}
+    return {b'node': node, b'oldphase': old, b'phase': phasenames[new]}
