@@ -15,6 +15,7 @@ import socket
 import sys
 
 from .i18n import _
+from .pycompat import getattr
 from . import (
     encoding,
     error,
@@ -25,15 +26,14 @@ from . import (
     urllibcompat,
     util,
 )
-from .utils import (
-    stringutil,
-)
+from .utils import stringutil
 
 httplib = util.httplib
 stringio = util.stringio
 urlerr = util.urlerr
 urlreq = util.urlreq
 
+
 def escape(s, quote=None):
     '''Replace special characters "&", "<" and ">" to HTML-safe sequences.
     If the optional flag quote is true, the quotation mark character (")
@@ -49,6 +49,7 @@ def escape(s, quote=None):
         s = s.replace(b'"', b"&quot;")
     return s
 
+
 class passwordmgr(object):
     def __init__(self, ui, passwddb):
         self.ui = ui
@@ -71,22 +72,26 @@ class passwordmgr(object):
             res = httpconnectionmod.readauthforuri(self.ui, authuri, user)
             if res:
                 group, auth = res
-                user, passwd = auth.get('username'), auth.get('password')
-                self.ui.debug("using auth.%s.* for authentication\n" % group)
+                user, passwd = auth.get(b'username'), auth.get(b'password')
+                self.ui.debug(b"using auth.%s.* for authentication\n" % group)
         if not user or not passwd:
             u = util.url(pycompat.bytesurl(authuri))
             u.query = None
             if not self.ui.interactive():
-                raise error.Abort(_('http authorization required for %s') %
-                                  util.hidepassword(bytes(u)))
+                raise error.Abort(
+                    _(b'http authorization required for %s')
+                    % util.hidepassword(bytes(u))
+                )
 
-            self.ui.write(_("http authorization required for %s\n") %
-                          util.hidepassword(bytes(u)))
-            self.ui.write(_("realm: %s\n") % pycompat.bytesurl(realm))
+            self.ui.write(
+                _(b"http authorization required for %s\n")
+                % util.hidepassword(bytes(u))
+            )
+            self.ui.write(_(b"realm: %s\n") % pycompat.bytesurl(realm))
             if user:
-                self.ui.write(_("user: %s\n") % user)
+                self.ui.write(_(b"user: %s\n") % user)
             else:
-                user = self.ui.prompt(_("user:"), default=None)
+                user = self.ui.prompt(_(b"user:"), default=None)
 
             if not passwd:
                 passwd = self.ui.getpass()
@@ -96,37 +101,45 @@ class passwordmgr(object):
         return (pycompat.strurl(user), pycompat.strurl(passwd))
 
     def _writedebug(self, user, passwd):
-        msg = _('http auth: user %s, password %s\n')
-        self.ui.debug(msg % (user, passwd and '*' * len(passwd) or 'not set'))
+        msg = _(b'http auth: user %s, password %s\n')
+        self.ui.debug(msg % (user, passwd and b'*' * len(passwd) or b'not set'))
 
     def find_stored_password(self, authuri):
         return self.passwddb.find_user_password(None, authuri)
 
+
 class proxyhandler(urlreq.proxyhandler):
     def __init__(self, ui):
-        proxyurl = (ui.config("http_proxy", "host") or
-                        encoding.environ.get('http_proxy'))
+        proxyurl = ui.config(b"http_proxy", b"host") or encoding.environ.get(
+            b'http_proxy'
+        )
         # XXX proxyauthinfo = None
 
         if proxyurl:
             # proxy can be proper url or host[:port]
-            if not (proxyurl.startswith('http:') or
-                    proxyurl.startswith('https:')):
-                proxyurl = 'http://' + proxyurl + '/'
+            if not (
+                proxyurl.startswith(b'http:') or proxyurl.startswith(b'https:')
+            ):
+                proxyurl = b'http://' + proxyurl + b'/'
             proxy = util.url(proxyurl)
             if not proxy.user:
-                proxy.user = ui.config("http_proxy", "user")
-                proxy.passwd = ui.config("http_proxy", "passwd")
+                proxy.user = ui.config(b"http_proxy", b"user")
+                proxy.passwd = ui.config(b"http_proxy", b"passwd")
 
             # see if we should use a proxy for this url
-            no_list = ["localhost", "127.0.0.1"]
-            no_list.extend([p.lower() for
-                            p in ui.configlist("http_proxy", "no")])
-            no_list.extend([p.strip().lower() for
-                            p in encoding.environ.get("no_proxy", '').split(',')
-                            if p.strip()])
+            no_list = [b"localhost", b"127.0.0.1"]
+            no_list.extend(
+                [p.lower() for p in ui.configlist(b"http_proxy", b"no")]
+            )
+            no_list.extend(
+                [
+                    p.strip().lower()
+                    for p in encoding.environ.get(b"no_proxy", b'').split(b',')
+                    if p.strip()
+                ]
+            )
             # "http_proxy.always" config is for running tests on localhost
-            if ui.configbool("http_proxy", "always"):
+            if ui.configbool(b"http_proxy", b"always"):
                 self.no_list = []
             else:
                 self.no_list = no_list
@@ -134,8 +147,8 @@ class proxyhandler(urlreq.proxyhandler):
             # Keys and values need to be str because the standard library
             # expects them to be.
             proxyurl = str(proxy)
-            proxies = {r'http': proxyurl, r'https': proxyurl}
-            ui.debug('proxying through %s\n' % util.hidepassword(bytes(proxy)))
+            proxies = {'http': proxyurl, 'https': proxyurl}
+            ui.debug(b'proxying through %s\n' % util.hidepassword(bytes(proxy)))
         else:
             proxies = {}
 
@@ -143,17 +156,18 @@ class proxyhandler(urlreq.proxyhandler):
         self.ui = ui
 
     def proxy_open(self, req, proxy, type_):
-        host = pycompat.bytesurl(urllibcompat.gethost(req)).split(':')[0]
+        host = pycompat.bytesurl(urllibcompat.gethost(req)).split(b':')[0]
         for e in self.no_list:
             if host == e:
                 return None
-            if e.startswith('*.') and host.endswith(e[2:]):
+            if e.startswith(b'*.') and host.endswith(e[2:]):
                 return None
-            if e.startswith('.') and host.endswith(e[1:]):
+            if e.startswith(b'.') and host.endswith(e[1:]):
                 return None
 
         return urlreq.proxyhandler.proxy_open(self, req, proxy, type_)
 
+
 def _gen_sendfile(orgsend):
     def _sendfile(self, data):
         # send a file
@@ -164,9 +178,12 @@ def _gen_sendfile(orgsend):
                 orgsend(self, chunk)
         else:
             orgsend(self, data)
+
     return _sendfile
 
-has_https = util.safehasattr(urlreq, 'httpshandler')
+
+has_https = util.safehasattr(urlreq, b'httpshandler')
+
 
 class httpconnection(keepalive.HTTPConnection):
     # must be able to send big bundle as stream.
@@ -181,22 +198,23 @@ class httpconnection(keepalive.HTTPConnection):
             return proxyres
         return keepalive.HTTPConnection.getresponse(self)
 
+
 # Large parts of this function have their origin from before Python 2.6
 # and could potentially be removed.
 def _generic_start_transaction(handler, h, req):
     tunnel_host = req._tunnel_host
     if tunnel_host:
-        if tunnel_host[:7] not in [r'http://', r'https:/']:
-            tunnel_host = r'https://' + tunnel_host
+        if tunnel_host[:7] not in ['http://', 'https:/']:
+            tunnel_host = 'https://' + tunnel_host
         new_tunnel = True
     else:
         tunnel_host = urllibcompat.getselector(req)
         new_tunnel = False
 
-    if new_tunnel or tunnel_host == urllibcompat.getfullurl(req): # has proxy
+    if new_tunnel or tunnel_host == urllibcompat.getfullurl(req):  # has proxy
         u = util.url(pycompat.bytesurl(tunnel_host))
-        if new_tunnel or u.scheme == 'https': # only use CONNECT for HTTPS
-            h.realhostport = ':'.join([u.host, (u.port or '443')])
+        if new_tunnel or u.scheme == b'https':  # only use CONNECT for HTTPS
+            h.realhostport = b':'.join([u.host, (u.port or b'443')])
             h.headers = req.headers.copy()
             h.headers.update(handler.parent.addheaders)
             return
@@ -204,14 +222,19 @@ def _generic_start_transaction(handler, h, req):
     h.realhostport = None
     h.headers = None
 
+
 def _generic_proxytunnel(self):
     proxyheaders = dict(
-            [(x, self.headers[x]) for x in self.headers
-             if x.lower().startswith(r'proxy-')])
-    self.send('CONNECT %s HTTP/1.0\r\n' % self.realhostport)
-    for header in proxyheaders.iteritems():
-        self.send('%s: %s\r\n' % header)
-    self.send('\r\n')
+        [
+            (x, self.headers[x])
+            for x in self.headers
+            if x.lower().startswith('proxy-')
+        ]
+    )
+    self.send(b'CONNECT %s HTTP/1.0\r\n' % self.realhostport)
+    for header in pycompat.iteritems(proxyheaders):
+        self.send(b'%s: %s\r\n' % header)
+    self.send(b'\r\n')
 
     # majority of the following code is duplicated from
     # httplib.HTTPConnection as there are no adequate places to
@@ -219,31 +242,29 @@ def _generic_proxytunnel(self):
     # strict was removed in Python 3.4.
     kwargs = {}
     if not pycompat.ispy3:
-        kwargs['strict'] = self.strict
+        kwargs[b'strict'] = self.strict
 
-    res = self.response_class(self.sock,
-                              method=self._method,
-                              **kwargs)
+    res = self.response_class(self.sock, method=self._method, **kwargs)
 
     while True:
         version, status, reason = res._read_status()
         if status != httplib.CONTINUE:
             break
         # skip lines that are all whitespace
-        list(iter(lambda: res.fp.readline().strip(), ''))
+        list(iter(lambda: res.fp.readline().strip(), b''))
     res.status = status
     res.reason = reason.strip()
 
     if res.status == 200:
         # skip lines until we find a blank line
-        list(iter(res.fp.readline, '\r\n'))
+        list(iter(res.fp.readline, b'\r\n'))
         return True
 
-    if version == 'HTTP/1.0':
+    if version == b'HTTP/1.0':
         res.version = 10
-    elif version.startswith('HTTP/1.'):
+    elif version.startswith(b'HTTP/1.'):
         res.version = 11
-    elif version == 'HTTP/0.9':
+    elif version == b'HTTP/0.9':
         res.version = 9
     else:
         raise httplib.UnknownProtocol(version)
@@ -259,8 +280,8 @@ def _generic_proxytunnel(self):
     res.msg.fp = None
 
     # are we using the chunked-style of transfer encoding?
-    trenc = res.msg.getheader('transfer-encoding')
-    if trenc and trenc.lower() == "chunked":
+    trenc = res.msg.getheader(b'transfer-encoding')
+    if trenc and trenc.lower() == b"chunked":
         res.chunked = 1
         res.chunk_left = None
     else:
@@ -272,7 +293,7 @@ def _generic_proxytunnel(self):
     # do we have a Content-Length?
     # NOTE: RFC 2616, section 4.4, #3 says we ignore this if
     # transfer-encoding is "chunked"
-    length = res.msg.getheader('content-length')
+    length = res.msg.getheader(b'content-length')
     if length and not res.chunked:
         try:
             res.length = int(length)
@@ -285,23 +306,25 @@ def _generic_proxytunnel(self):
         res.length = None
 
     # does the body have a fixed length? (of zero)
-    if (status == httplib.NO_CONTENT or status == httplib.NOT_MODIFIED or
-        100 <= status < 200 or # 1xx codes
-        res._method == 'HEAD'):
+    if (
+        status == httplib.NO_CONTENT
+        or status == httplib.NOT_MODIFIED
+        or 100 <= status < 200
+        or res._method == b'HEAD'  # 1xx codes
+    ):
         res.length = 0
 
     # if the connection remains open, and we aren't using chunked, and
     # a content-length was not provided, then assume that the connection
     # WILL close.
-    if (not res.will_close and
-       not res.chunked and
-       res.length is None):
+    if not res.will_close and not res.chunked and res.length is None:
         res.will_close = 1
 
     self.proxyres = res
 
     return False
 
+
 class httphandler(keepalive.HTTPHandler):
     def http_open(self, req):
         return self.do_open(httpconnection, req)
@@ -310,6 +333,7 @@ class httphandler(keepalive.HTTPHandler):
         _generic_start_transaction(self, h, req)
         return keepalive.HTTPHandler._start_transaction(self, h, req)
 
+
 class logginghttpconnection(keepalive.HTTPConnection):
     def __init__(self, createconn, *args, **kwargs):
         keepalive.HTTPConnection.__init__(self, *args, **kwargs)
@@ -319,14 +343,16 @@ class logginghttpconnection(keepalive.HTTPConnection):
         # copied from 2.7.14, since old implementations directly call
         # socket.create_connection()
         def connect(self):
-            self.sock = self._create_connection((self.host, self.port),
-                                                self.timeout,
-                                                self.source_address)
+            self.sock = self._create_connection(
+                (self.host, self.port), self.timeout, self.source_address
+            )
             if self._tunnel_host:
                 self._tunnel()
 
+
 class logginghttphandler(httphandler):
     """HTTP handler that logs socket I/O."""
+
     def __init__(self, logfh, name, observeropts, timeout=None):
         super(logginghttphandler, self).__init__(timeout=timeout)
 
@@ -343,12 +369,15 @@ class logginghttphandler(httphandler):
     def _makeconnection(self, *args, **kwargs):
         def createconnection(*args, **kwargs):
             sock = socket.create_connection(*args, **kwargs)
-            return util.makeloggingsocket(self._logfh, sock, self._logname,
-                                          **self._observeropts)
+            return util.makeloggingsocket(
+                self._logfh, sock, self._logname, **self._observeropts
+            )
 
         return logginghttpconnection(createconnection, *args, **kwargs)
 
+
 if has_https:
+
     class httpsconnection(keepalive.HTTPConnection):
         response_class = keepalive.HTTPResponse
         default_port = httplib.HTTPS_PORT
@@ -356,8 +385,15 @@ if has_https:
         send = _gen_sendfile(keepalive.safesend)
         getresponse = keepalive.wrapgetresponse(httplib.HTTPConnection)
 
-        def __init__(self, host, port=None, key_file=None, cert_file=None,
-                     *args, **kwargs):
+        def __init__(
+            self,
+            host,
+            port=None,
+            key_file=None,
+            cert_file=None,
+            *args,
+            **kwargs
+        ):
             keepalive.HTTPConnection.__init__(self, host, port, *args, **kwargs)
             self.key_file = key_file
             self.cert_file = cert_file
@@ -366,12 +402,16 @@ if has_https:
             self.sock = socket.create_connection((self.host, self.port))
 
             host = self.host
-            if self.realhostport: # use CONNECT proxy
+            if self.realhostport:  # use CONNECT proxy
                 _generic_proxytunnel(self)
-                host = self.realhostport.rsplit(':', 1)[0]
+                host = self.realhostport.rsplit(b':', 1)[0]
             self.sock = sslutil.wrapsocket(
-                self.sock, self.key_file, self.cert_file, ui=self.ui,
-                serverhostname=host)
+                self.sock,
+                self.key_file,
+                self.cert_file,
+                ui=self.ui,
+                serverhostname=host,
+            )
             sslutil.validatesocket(self.sock)
 
     class httpshandler(keepalive.KeepAliveHandler, urlreq.httpshandler):
@@ -379,8 +419,7 @@ if has_https:
             keepalive.KeepAliveHandler.__init__(self, timeout=timeout)
             urlreq.httpshandler.__init__(self)
             self.ui = ui
-            self.pwmgr = passwordmgr(self.ui,
-                                     self.ui.httppasswordmgrdb)
+            self.pwmgr = passwordmgr(self.ui, self.ui.httppasswordmgrdb)
 
         def _start_transaction(self, h, req):
             _generic_start_transaction(self, h, req)
@@ -395,7 +434,7 @@ if has_https:
             if res:
                 group, auth = res
                 self.auth = auth
-                self.ui.debug("using auth.%s.* for authentication\n" % group)
+                self.ui.debug(b"using auth.%s.* for authentication\n" % group)
             else:
                 self.auth = None
             return self.do_open(self._makeconnection, req)
@@ -404,23 +443,25 @@ if has_https:
             keyfile = None
             certfile = None
 
-            if len(args) >= 1: # key_file
+            if len(args) >= 1:  # key_file
                 keyfile = args[0]
-            if len(args) >= 2: # cert_file
+            if len(args) >= 2:  # cert_file
                 certfile = args[1]
             args = args[2:]
 
             # if the user has specified different key/cert files in
             # hgrc, we prefer these
-            if self.auth and 'key' in self.auth and 'cert' in self.auth:
-                keyfile = self.auth['key']
-                certfile = self.auth['cert']
+            if self.auth and b'key' in self.auth and b'cert' in self.auth:
+                keyfile = self.auth[b'key']
+                certfile = self.auth[b'cert']
 
-            conn = httpsconnection(host, port, keyfile, certfile, *args,
-                                   **kwargs)
+            conn = httpsconnection(
+                host, port, keyfile, certfile, *args, **kwargs
+            )
             conn.ui = self.ui
             return conn
 
+
 class httpdigestauthhandler(urlreq.httpdigestauthhandler):
     def __init__(self, *args, **kwargs):
         urlreq.httpdigestauthhandler.__init__(self, *args, **kwargs)
@@ -438,7 +479,9 @@ class httpdigestauthhandler(urlreq.httpdigestauthhandler):
             self.retried_req = req
             self.retried = 0
         return urlreq.httpdigestauthhandler.http_error_auth_reqed(
-                    self, auth_header, host, req, headers)
+            self, auth_header, host, req, headers
+        )
+
 
 class httpbasicauthhandler(urlreq.httpbasicauthhandler):
     def __init__(self, *args, **kwargs):
@@ -470,14 +513,16 @@ class httpbasicauthhandler(urlreq.httpbasicauthhandler):
             self.retried_req = req
             self.retried = 0
         return urlreq.httpbasicauthhandler.http_error_auth_reqed(
-                        self, auth_header, host, req, headers)
+            self, auth_header, host, req, headers
+        )
 
     def retry_http_basic_auth(self, host, req, realm):
         user, pw = self.passwd.find_user_password(
-            realm, urllibcompat.getfullurl(req))
+            realm, urllibcompat.getfullurl(req)
+        )
         if pw is not None:
-            raw = "%s:%s" % (pycompat.bytesurl(user), pycompat.bytesurl(pw))
-            auth = r'Basic %s' % pycompat.strurl(base64.b64encode(raw).strip())
+            raw = b"%s:%s" % (pycompat.bytesurl(user), pycompat.bytesurl(pw))
+            auth = 'Basic %s' % pycompat.strurl(base64.b64encode(raw).strip())
             if req.get_header(self.auth_header, None) == auth:
                 return None
             self.auth = auth
@@ -486,23 +531,30 @@ class httpbasicauthhandler(urlreq.httpbasicauthhandler):
         else:
             return None
 
+
 class cookiehandler(urlreq.basehandler):
     def __init__(self, ui):
         self.cookiejar = None
 
-        cookiefile = ui.config('auth', 'cookiefile')
+        cookiefile = ui.config(b'auth', b'cookiefile')
         if not cookiefile:
             return
 
         cookiefile = util.expandpath(cookiefile)
         try:
             cookiejar = util.cookielib.MozillaCookieJar(
-                pycompat.fsdecode(cookiefile))
+                pycompat.fsdecode(cookiefile)
+            )
             cookiejar.load()
             self.cookiejar = cookiejar
         except util.cookielib.LoadError as e:
-            ui.warn(_('(error loading cookie file %s: %s; continuing without '
-                      'cookies)\n') % (cookiefile, stringutil.forcebytestr(e)))
+            ui.warn(
+                _(
+                    b'(error loading cookie file %s: %s; continuing without '
+                    b'cookies)\n'
+                )
+                % (cookiefile, stringutil.forcebytestr(e))
+            )
 
     def http_request(self, request):
         if self.cookiejar:
@@ -516,10 +568,19 @@ class cookiehandler(urlreq.basehandler):
 
         return request
 
+
 handlerfuncs = []
 
-def opener(ui, authinfo=None, useragent=None, loggingfh=None,
-           loggingname=b's', loggingopts=None, sendaccept=True):
+
+def opener(
+    ui,
+    authinfo=None,
+    useragent=None,
+    loggingfh=None,
+    loggingname=b's',
+    loggingopts=None,
+    sendaccept=True,
+):
     '''
     construct an opener suitable for urllib2
     authinfo will be added to the password manager
@@ -535,12 +596,15 @@ def opener(ui, authinfo=None, useragent=None, loggingfh=None,
     ``sendaccept`` allows controlling whether the ``Accept`` request header
     is sent. The header is sent by default.
     '''
-    timeout = ui.configwith(float, 'http', 'timeout')
+    timeout = ui.configwith(float, b'http', b'timeout')
     handlers = []
 
     if loggingfh:
-        handlers.append(logginghttphandler(loggingfh, loggingname,
-                                           loggingopts or {}, timeout=timeout))
+        handlers.append(
+            logginghttphandler(
+                loggingfh, loggingname, loggingopts or {}, timeout=timeout
+            )
+        )
         # We don't yet support HTTPS when logging I/O. If we attempt to open
         # an HTTPS URL, we'll likely fail due to unknown protocol.
 
@@ -557,11 +621,14 @@ def opener(ui, authinfo=None, useragent=None, loggingfh=None,
         saveduser, savedpass = passmgr.find_stored_password(uris[0])
         if user != saveduser or passwd:
             passmgr.add_password(realm, uris, user, passwd)
-        ui.debug('http auth: user %s, password %s\n' %
-                 (user, passwd and '*' * len(passwd) or 'not set'))
+        ui.debug(
+            b'http auth: user %s, password %s\n'
+            % (user, passwd and b'*' * len(passwd) or b'not set')
+        )
 
-    handlers.extend((httpbasicauthhandler(passmgr),
-                     httpdigestauthhandler(passmgr)))
+    handlers.extend(
+        (httpbasicauthhandler(passmgr), httpdigestauthhandler(passmgr))
+    )
     handlers.extend([h(ui, passmgr) for h in handlerfuncs])
     handlers.append(cookiehandler(ui))
     opener = urlreq.buildopener(*handlers)
@@ -587,20 +654,21 @@ def opener(ui, authinfo=None, useragent=None, loggingfh=None,
     # The custom user agent is for lfs, because unfortunately some servers
     # do look at this value.
     if not useragent:
-        agent = 'mercurial/proto-1.0 (Mercurial %s)' % util.version()
-        opener.addheaders = [(r'User-agent', pycompat.sysstr(agent))]
+        agent = b'mercurial/proto-1.0 (Mercurial %s)' % util.version()
+        opener.addheaders = [('User-agent', pycompat.sysstr(agent))]
     else:
-        opener.addheaders = [(r'User-agent', pycompat.sysstr(useragent))]
+        opener.addheaders = [('User-agent', pycompat.sysstr(useragent))]
 
     # This header should only be needed by wire protocol requests. But it has
     # been sent on all requests since forever. We keep sending it for backwards
     # compatibility reasons. Modern versions of the wire protocol use
     # X-HgProto-<N> for advertising client support.
     if sendaccept:
-        opener.addheaders.append((r'Accept', r'application/mercurial-0.1'))
+        opener.addheaders.append(('Accept', 'application/mercurial-0.1'))
 
     return opener
 
+
 def open(ui, url_, data=None, sendaccept=True):
     u = util.url(url_)
     if u.scheme:
@@ -608,11 +676,12 @@ def open(ui, url_, data=None, sendaccept=True):
         url_, authinfo = u.authinfo()
     else:
         path = util.normpath(os.path.abspath(url_))
-        url_ = 'file://' + pycompat.bytesurl(urlreq.pathname2url(path))
+        url_ = b'file://' + pycompat.bytesurl(urlreq.pathname2url(path))
         authinfo = None
-    return opener(ui, authinfo,
-                  sendaccept=sendaccept).open(pycompat.strurl(url_),
-                                              data)
+    return opener(ui, authinfo, sendaccept=sendaccept).open(
+        pycompat.strurl(url_), data
+    )
+
 
 def wrapresponse(resp):
     """Wrap a response object with common error handlers.
@@ -631,21 +700,29 @@ def wrapresponse(resp):
                 if e.expected:
                     got = len(e.partial)
                     total = e.expected + got
-                    msg = _('HTTP request error (incomplete response; '
-                            'expected %d bytes got %d)') % (total, got)
+                    msg = _(
+                        b'HTTP request error (incomplete response; '
+                        b'expected %d bytes got %d)'
+                    ) % (total, got)
                 else:
-                    msg = _('HTTP request error (incomplete response)')
+                    msg = _(b'HTTP request error (incomplete response)')
 
                 raise error.PeerTransportError(
                     msg,
-                    hint=_('this may be an intermittent network failure; '
-                           'if the error persists, consider contacting the '
-                           'network or server operator'))
+                    hint=_(
+                        b'this may be an intermittent network failure; '
+                        b'if the error persists, consider contacting the '
+                        b'network or server operator'
+                    ),
+                )
             except httplib.HTTPException as e:
                 raise error.PeerTransportError(
-                    _('HTTP request error (%s)') % e,
-                    hint=_('this may be an intermittent network failure; '
-                           'if the error persists, consider contacting the '
-                           'network or server operator'))
+                    _(b'HTTP request error (%s)') % e,
+                    hint=_(
+                        b'this may be an intermittent network failure; '
+                        b'if the error persists, consider contacting the '
+                        b'network or server operator'
+                    ),
+                )
 
     resp.__class__ = readerproxy
